ACCESS_TOKEN = "access_token"
ACCESS_TOKEN_EXPIRE_IN = "expires_in"
ACCESS_TOKEN_ISSUED_AT = "issued_at"
AUTHORIZATION_CODE = "code"
STATE = "state"
ID_TOKEN = "id_token"
ID_TOKEN_JWT = "id_token_jwt"
OIDC_SCOPE = "openid"
PKCE_CODE_VERIFIER = "pkce_code_verifier"
REFRESH_TOKEN = "refresh_token"
SCOPE = "scope"
TOKEN_TYPE = "token_type"
REQUEST_PARAMS = "request_params"
ISSUER = "issuer"
