AUTHORIZATION_CODE_TYPE = 'authorization_code'
BASIC_TYPE = 'basic'

LOGIN_SCOPE = 'internal_login'
HUMAN_TASK_SCOPE = 'internal_humantask_view'
DEFAULT_SUPER_TENANT = 'carbon.super'
ISSUER = 'issuer'

CODE = 'code'
TRUE_STRING = 'true'

AUTHORIZATION_ENDPOINT = 'authorization_endpoint'
TOKEN_ENDPOINT = 'token_endpoint'
REVOKE_TOKEN_ENDPOINT = 'revoke_token_endpoint'
END_SESSION_ENDPOINT = 'end_session_endpoint'
JWKS_ENDPOINT = 'jwks_uri'
OP_CONFIG_INITIATED = 'op_config_initiated'
TENANT = 'tenant'

BASE_URI = 'https://localhost:9443'
AUTH_URI = '/oauth2/authorize'
DEVICE_URI = 'https://oauth2.googleapis.com/device/code'
REVOKE_URI = 'https://oauth2.googleapis.com/revoke'
TOKEN_URI = '/oauth2/token'
TOKEN_INFO_URI = 'https://oauth2.googleapis.com/tokeninfo'

REDIRECT = 'redirect'
USER = 'user'
TOKEN_RESPONSE = 'token_response'
URL = 'url'
CERTIFICATE_PATH = "certificate_path"
